'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var _regeneratorRuntime = _interopDefault(require('babel-runtime/regenerator'));
var _slicedToArray = _interopDefault(require('babel-runtime/helpers/slicedToArray'));
var _asyncToGenerator = _interopDefault(require('babel-runtime/helpers/asyncToGenerator'));
var rollupPluginutils = require('rollup-pluginutils');
var path = require('path');
var fs = require('fs');
var _Promise = _interopDefault(require('babel-runtime/core-js/promise'));

function promisify(nodeFn) {
  return function () {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return new _Promise(function (resolve, reject) {
      nodeFn.apply(undefined, args.concat([function (error, result) {
        if (error) {
          reject(error);
        } else {
          resolve(result);
        }
      }]));
    });
  };
}

var readFileAsync = promisify(fs.readFile);

// Borrowed from https://github.com/webpack/source-map-loader
var baseRegex = '\\s*[@#]\\s*sourceMappingURL\\s*=\\s*([^\\s]*)';
var regex1 = new RegExp('/\\*' + baseRegex + '\\s*\\*/'); // Matches /* ... */ comments
var regex2 = new RegExp('//' + baseRegex + '($|\n|\r\n?)'); // Matches // .... comments
var regexDataUrl = /data:[^;\n]+;base64,(.*)/; // Matches DataUrls

function sourceMapsPlugin() {
  var _ref = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  var include = _ref.include;
  var exclude = _ref.exclude;

  var filter = rollupPluginutils.createFilter(include, exclude);

  return {
    load: function load(id) {
      var _this = this;

      return _asyncToGenerator(_regeneratorRuntime.mark(function _callee() {
        var code, match, _match, sourceMapComment, sourceMapURL, dataUrlMatch, context, rawMap, fileName, map;

        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                if (filter(id)) {
                  _context.next = 2;
                  break;
                }

                return _context.abrupt('return', null);

              case 2:
                code = void 0;
                _context.prev = 3;
                _context.next = 6;
                return readFileAsync(id, 'utf8');

              case 6:
                code = _context.sent;
                _context.next = 12;
                break;

              case 9:
                _context.prev = 9;
                _context.t0 = _context['catch'](3);
                return _context.abrupt('return', null);

              case 12:
                match = regex1.exec(code) || regex2.exec(code);

                if (!(match === null)) {
                  _context.next = 15;
                  break;
                }

                return _context.abrupt('return', code);

              case 15:
                _match = _slicedToArray(match, 2);
                sourceMapComment = _match[0];
                sourceMapURL = _match[1];
                dataUrlMatch = regexDataUrl.exec(sourceMapURL);
                context = path.dirname(id);

                // Remove the source map comment, and make sure we don't return an empty string

                code = code.replace(sourceMapComment, '') || '\n';

                rawMap = void 0;

                if (!dataUrlMatch) {
                  _context.next = 26;
                  break;
                }

                rawMap = new Buffer(dataUrlMatch[1], 'base64').toString();
                _context.next = 37;
                break;

              case 26:
                fileName = path.resolve(context, sourceMapURL);
                _context.prev = 27;
                _context.next = 30;
                return readFileAsync(fileName, 'utf8');

              case 30:
                rawMap = _context.sent;
                _context.next = 37;
                break;

              case 33:
                _context.prev = 33;
                _context.t1 = _context['catch'](27);

                console.error('rollup-plugin-sourcemaps: Could not open source map: ' + _context.t1);
                return _context.abrupt('return', code);

              case 37:
                map = void 0;
                _context.prev = 38;

                map = JSON.parse(rawMap);
                _context.next = 46;
                break;

              case 42:
                _context.prev = 42;
                _context.t2 = _context['catch'](38);

                console.error('rollup-plugin-sourcemaps: Failed parsing raw source map from ' + id + ': ' + _context.t2);
                return _context.abrupt('return', code);

              case 46:
                return _context.abrupt('return', { code: code, map: map });

              case 47:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, _this, [[3, 9], [27, 33], [38, 42]]);
      }))();
    }
  };
}

module.exports = sourceMapsPlugin;
//# sourceMappingURL=rollup-plugin-sourcemaps.js.map